From d6d6ba0fc4f1f976bc185cc773f9b4dfd0930403 Mon Sep 17 00:00:00 2001
From: Shlok Kyal <shlok.kyal.oss@gmail.com>
Date: Sat, 10 May 2025 23:28:09 +0530
Subject: [PATCH v3_REL_17] Fix duplicate insert during pg_createsubscriber

When pg_createsubscriber is run, the standby node is recovered to a
point 'consistent_lsn' and promoted. Then, when the subscription is
created, the replication origin is advanced to the 'consistent_lsn'.
Then the subscription is enabled and the apply worker starts to send
changes from 'consistent_lsn'.

When this 'consistent_lsn' is an LSN corresponding to a COMMIT, the
records for the transaction for that COMMIT are already replicated to
the standby node during the recovery phase. Now, when the subscription
is created, the replication origin is advanced and the subscription is
enabled. The apply worker starts to apply changes from 'consistent_lsn'.
So, records corresponding to the transaction whose COMMIT LSN is
'consistent_lsn' are replicated again.

To avoid this, set recovery_target_inclusive = false instead of true.
So the standby will recovery till point just before 'consistent_lsn'.
And apply worker will start(after enabling of subscription) to apply
changes from 'consistent_lsn'. So, above scenorio is avoided.
---
 src/bin/pg_basebackup/pg_createsubscriber.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/src/bin/pg_basebackup/pg_createsubscriber.c b/src/bin/pg_basebackup/pg_createsubscriber.c
index 677c0cd0843..1003c329c73 100644
--- a/src/bin/pg_basebackup/pg_createsubscriber.c
+++ b/src/bin/pg_basebackup/pg_createsubscriber.c
@@ -1207,7 +1207,7 @@ setup_recovery(const struct LogicalRepInfo *dbinfo, const char *datadir, const c
 	appendPQExpBuffer(recoveryconfcontents,
 					  "recovery_target_timeline = 'latest'\n");
 	appendPQExpBuffer(recoveryconfcontents,
-					  "recovery_target_inclusive = true\n");
+					  "recovery_target_inclusive = false\n");
 	appendPQExpBuffer(recoveryconfcontents,
 					  "recovery_target_action = promote\n");
 	appendPQExpBuffer(recoveryconfcontents, "recovery_target_name = ''\n");
-- 
2.34.1

